import os

import discretisedfield as df
import micromagneticmodel as mm


def test_outputstep(calculator):
    name = "output_step"

    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    n = (2, 2, 2)
    Ms = 1e6
    A = 1e-12
    H = (0, 0, 1e6)
    region = df.Region(p1=p1, p2=p2)
    mesh = df.Mesh(region=region, n=n)

    system = mm.System(name=name)
    system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
    system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

    md = calculator.MinDriver()
    md.drive(system, output_step=True)

    dirname = os.path.join(name, "drive-0")
    assert os.path.exists(dirname)

    assert len(system.table.data.index) > 1

    calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestSlonczewski:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-5e-9, -5e-9, -3e-9)
        p2 = (5e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.n = (2, 2, 2)
        self.subregions = {
            "r1": df.Region(p1=(-5e-9, -5e-9, -3e-9), p2=(5e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-5e-9, 0, -3e-9), p2=(5e-9, 5e-9, 3e-9)),
        }

    def test_single_values(self):
        name = "slonczewski_scalar_values"

        J = 1e12
        mp = (1, 0, 0)
        P = 0.4
        Lambda = 2
        eps_prime = 0
        H = (0, 0, 1e6)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, n=self.n)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=20)

        # Check if it runs.

        # remove current -> needs different evolver
        system.dynamics -= mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime
        )

        td.drive(system, t=0.2e-9, n=20)

        # Check if it runs.

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, dt=1e-13, func=time_dep
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, tcl_strings=tcl_strings
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        self.calculator.delete(system)

    def test_single_values_finite_temperature(self):
        name = "slonczewski_scalar_values_finite_temperature"

        J = 1e12
        mp = (1, 0, 0)
        P = 0.4
        Lambda = 2
        eps_prime = 0
        H = (0, 0, 1e6)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, n=self.n)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)
        system.T = 10

        evolver = self.calculator.Xf_ThermSpinXferEvolver()
        td = self.calculator.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=20)

        # Check if it runs.

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, dt=1e-13, func=time_dep
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, tcl_strings=tcl_strings
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        self.calculator.delete(system)

    def test_dict_values(self):
        name = "slonczewski_scalar_values"

        J = {"r1": 1e12, "r2": 5e12}
        mp = {"r1": (0, 0, 1), "r2": (0, 1, 0)}
        P = {"r1": 0.4, "r2": 0.35}
        Lambda = {"r1": 2, "r2": 1.5}
        eps_prime = {"r1": 0, "r2": 1}
        H = (0, 0, 1e6)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, n=self.n, subregions=self.subregions)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=20)

        # Check if it runs.

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, dt=1e-13, func=time_dep
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, tcl_strings=tcl_strings
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        self.calculator.delete(system)

    def test_field_values(self):
        name = "slonczewski_scalar_values"

        mesh = df.Mesh(region=self.region, n=self.n)

        J = df.Field(mesh, nvdim=1, value=0.5e12)
        mp = df.Field(mesh, nvdim=3, value=(1, 0, 0))
        P = df.Field(mesh, nvdim=1, value=0.5)
        Lambda = df.Field(mesh, nvdim=1, value=2)
        eps_prime = df.Field(mesh, nvdim=1, value=1)
        H = (0, 0, 1e6)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=20)

        # Check if it runs.

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, dt=1e-13, func=time_dep
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.Slonczewski(
            J=J, mp=mp, P=P, Lambda=Lambda, eps_prime=eps_prime, tcl_strings=tcl_strings
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestMesh:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-7e-9, -5e-9, -4e-9)
        p2 = (7e-9, 5e-9, 4e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.bc = "xyz"
        self.subregions = {
            "r1": df.Region(p1=(-7e-9, -5e-9, -4e-9), p2=(7e-9, 0, 4e-9)),
            "r2": df.Region(p1=(-7e-9, 0, -4e-9), p2=(7e-9, 2e-9, 4e-9)),
            "r3": df.Region(p1=(-7e-9, 2e-9, -4e-9), p2=(7e-9, 5e-9, 4e-9)),
        }

    def test_single_nopbc(self):
        name = "mesh_single_nopbc"

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = df.Mesh(region=self.region, cell=self.cell)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(1, 0, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        self.calculator.delete(system)

    def test_multi_nopbc(self):
        name = "mesh_multi_nopbc"

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(1, 0, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        self.calculator.delete(system)

    def test_single_pbc(self):
        name = "mesh_single_pbc"

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = df.Mesh(region=self.region, cell=self.cell, bc=self.bc)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(1, 0, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        self.calculator.delete(system)

    def test_multi_pbc(self):
        name = "mesh_multi_pbc"

        Ms = 1e6
        H = (0, 0, 5e6)

        mesh = df.Mesh(
            region=self.region, cell=self.cell, bc=self.bc, subregions=self.subregions
        )

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(1, 0, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        self.calculator.delete(system)



import json
import os
import re

import discretisedfield as df
import micromagneticmodel as mm


def test_info_file(calculator):
    name = "info_file"

    L = 30e-9  # (m)
    cell = (10e-9, 15e-9, 5e-9)  # (m)
    A = 1.3e-11  # (J/m)
    Ms = 8e5  # (A/m)
    H = (1e6, 0.0, 2e5)  # (A/m)
    gamma0 = 2.211e5  # (m/As)
    alpha = 0.02

    region = df.Region(p1=(0, 0, 0), p2=(L, L, L))
    mesh = df.Mesh(region=region, cell=cell)
    system = mm.System(name=name)
    system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
    system.dynamics = mm.Precession(gamma0=gamma0) + mm.Damping(alpha=alpha)
    system.m = df.Field(mesh, nvdim=3, value=(0.0, 0.25, 0.1), norm=Ms)

    # First (0) drive
    td = calculator.TimeDriver()
    td.drive(system, t=25e-12, n=10)

    dirname = os.path.join(name, "drive-0")
    infofile = os.path.join(dirname, "info.json")
    assert os.path.exists(dirname)
    assert os.path.isfile(infofile)

    with open(infofile) as f:
        info = json.loads(f.read())
    assert "drive_number" in info.keys()
    assert "date" in info.keys()
    assert "time" in info.keys()
    assert "driver" in info.keys()

    assert info["drive_number"] == 0
    assert re.findall(r"\d{4}-\d{2}-\d{2}", info["date"]) is not []
    assert re.findall(r"\d{2}:\d{2}-\d{2}", info["time"]) is not []
    assert info["driver"] == "TimeDriver"
    assert info["t"] == 25e-12
    assert info["n"] == 10

    # Second (1) drive
    md = calculator.MinDriver()
    md.drive(system)

    dirname = os.path.join(name, "drive-1")
    infofile = os.path.join(dirname, "info.json")
    assert os.path.exists(dirname)
    assert os.path.isfile(infofile)

    with open(infofile) as f:
        info = json.loads(f.read())
    assert "drive_number" in info.keys()
    assert "date" in info.keys()
    assert "time" in info.keys()
    assert "driver" in info.keys()

    assert info["drive_number"] == 1
    assert re.findall(r"\d{4}-\d{2}-\d{2}", info["date"]) is not []
    assert re.findall(r"\d{2}:\d{2}-\d{2}", info["time"]) is not []
    assert info["driver"] == "MinDriver"

    calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm


def test_skyrmion(calculator):
    name = "skyrmion"

    Ms = 1.1e6
    A = 1.6e-11
    D = 4e-3
    K = 0.51e6
    u = (0, 0, 1)
    H = (0, 0, 2e5)

    p1 = (-50e-9, -50e-9, 0)
    p2 = (50e-9, 50e-9, 10e-9)
    cell = (5e-9, 5e-9, 5e-9)
    mesh = df.Mesh(p1=p1, p2=p2, cell=cell)

    system = mm.System(name=name)
    system.energy = (
        mm.Exchange(A=A)
        + mm.DMI(D=D, crystalclass="Cnv_z")
        + mm.UniaxialAnisotropy(K=K, u=u)
        + mm.Demag()
        + mm.Zeeman(H=H)
    )

    def Ms_fun(pos):
        x, y, z = pos
        if (x**2 + y**2) ** 0.5 < 50e-9:
            return Ms
        else:
            return 0

    def m_init(pos):
        x, y, z = pos
        if (x**2 + y**2) ** 0.5 < 10e-9:
            return (0, 0.1, -1)
        else:
            return (0, 0.1, 1)

    system.m = df.Field(mesh, nvdim=3, value=m_init, norm=Ms_fun)

    md = calculator.MinDriver()
    md.drive(system)

    # Check the magnetisation at the sample centre.
    value = system.m((0, 0, 0))
    assert value[2] / Ms < -0.97

    # Check the magnetisation at the sample edge.
    value = system.m((50e-9, 0, 0))
    assert value[2] / Ms > 0.5

    calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import pytest


def test_format(calculator):
    name = "output_format"

    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    n = (2, 2, 2)
    Ms = 1e6
    A = 1e-12
    H = (0, 0, 1e6)
    region = df.Region(p1=p1, p2=p2)
    mesh = df.Mesh(region=region, n=n)

    system = mm.System(name=name)
    system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
    system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

    md = calculator.MinDriver()

    # test if it runs
    md.drive(system)  # 'bin8' (default)
    md.drive(system, ovf_format="bin4")
    md.drive(system, ovf_format="txt")
    with pytest.raises(ValueError):
        md.drive(system, ovf_format="unknown")

    calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestEnergy:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (0, 0, 0)
        p2 = (10e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.subregions = {
            "r1": df.Region(p1=(0, 0, 0), p2=(5e-9, 5e-9, 3e-9)),
            "r2": df.Region(p1=(5e-9, 0, 0), p2=(10e-9, 5e-9, 3e-9)),
        }

    def test_exchange_zeeman(self):
        name = "energy_exchange_zeeman"

        A = 1e-12
        H = (1e6, 0, 0)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, cell=self.cell)

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(0, 1, 0), norm=Ms)

        if hasattr(self.calculator, "RelaxDriver"):
            system.dynamics = mm.Damping(alpha=0.5)
            md = self.calculator.RelaxDriver()
        else:
            md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        self.calculator.delete(system)

    def test_exchange_uniaxialanisotropy(self):
        name = "exchange_uniaxialanisotropy"

        A = {"r1": 1e-12, "r2": 0}
        K = 1e5
        u = (1, 0, 0)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A) + mm.UniaxialAnisotropy(K=K, u=u)
        system.m = df.Field(mesh, nvdim=3, value=(0.5, 1, 0), norm=Ms)

        if hasattr(self.calculator, "RelaxDriver"):
            system.dynamics = mm.Damping(alpha=0.5)
            md = self.calculator.RelaxDriver()
        else:
            md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        self.calculator.delete(system)

    def test_exchange_cubicanisotropy(self):
        name = "exchange_cubicanisotropy"

        A = {"r1": 1e-12, "r2": 0}
        K = 1e5
        u1 = (1, 0, 0)
        u2 = (0, 1, 0)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A) + mm.CubicAnisotropy(K=K, u1=u1, u2=u2)
        system.m = df.Field(mesh, nvdim=3, value=(1, 0.3, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        self.calculator.delete(system)

    def test_exchange_dmi_zeeman(self):
        name = "exchange_dmi_zeeman"

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)

        # Very weak DMI and strong Zeeman to make the final
        # magnetisation uniform.
        A = {"r1": 1e-12, "r2": 3e-12, "r1:r2": 2e-12}
        D = {"r1": 1e-9, "r2": 0, "r1:r2": 5e-9}
        H = df.Field(mesh, nvdim=3, value=(1e10, 0, 0))
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = (
            mm.Exchange(A=A) + mm.DMI(D=D, crystalclass="Cnv_z") + mm.Zeeman(H=H)
        )
        system.m = df.Field(mesh, nvdim=3, value=(1, 0.3, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1

        self.calculator.delete(system)

    def test_exchange_dmi_zeeman_uniaxialanisotropy_demag(self):
        name = "exchange_dmi_zeeman_uniaxialanisotropy"

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)

        # Very weak DMI and strong Zeeman to make the final
        # magnetisation uniform.
        A = {"r1": 1e-12, "r2": 3e-12, "r1:r2": 2e-12}
        D = {"r1": 1e-9, "r2": 0, "r1:r2": 5e-9}  # Very weak DMI
        H = df.Field(mesh, nvdim=3, value=(1e12, 0, 0))
        K = 1e6
        u = (1, 0, 0)
        Ms = 1e5

        system = mm.System(name=name)
        system.energy = (
            mm.Exchange(A=A)
            + mm.DMI(D=D, crystalclass="Cnv_z")
            + mm.UniaxialAnisotropy(K=K, u=u)
            + mm.Zeeman(H=H)
            + mm.Demag()
        )
        system.m = df.Field(mesh, nvdim=3, value=(1, 0.3, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1

        self.calculator.delete(system)

    def test_zeeman_zeeman(self):
        name = "multi_zeeman_field"

        def value_fun(pos):
            x, _, _ = pos
            if x <= 0:
                return (1e6, 0, 0)
            else:
                return (0, 0, 1e6)

        mesh = df.Mesh(region=self.region, cell=self.cell)

        H1 = df.Field(mesh, nvdim=3, value=value_fun)
        H2 = df.Field(mesh, nvdim=3, value=(0, 1e6, 0))
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H1, name="zeeman1") + mm.Zeeman(
            H=H2, name="zeeman2"
        )
        system.m = df.Field(mesh, nvdim=3, value=(0, 1, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        # test if it runs

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
from scipy.optimize import bisect  # This is why scipy is a dependency.


def test_stdprob3(calculator):
    name = "stdprob3"

    # Function for initiaising the flower state.
    def m_init_flower(pos):
        _, y, z = pos[0] / 1e-9, pos[1] / 1e-9, pos[2] / 1e-9
        mx = 0
        my = 2 * z - 1
        mz = -2 * y + 1
        norm_squared = mx**2 + my**2 + mz**2
        if norm_squared <= 0.05:
            return (1, 0, 0)
        else:
            return (mx, my, mz)

    # Function for initialising the vortex state.
    def m_init_vortex(pos):
        x, _, _ = pos[0] / 1e-9, pos[1] / 1e-9, pos[2] / 1e-9
        mx = 0
        my = np.sin(np.pi / 2 * (x - 0.5))
        mz = np.cos(np.pi / 2 * (x - 0.5))

        return (mx, my, mz)

    def minimise_system_energy(L, m_init):
        N = 16  # discretisation in one nvdimension
        cubesize = 100e-9  # cube edge length (m)
        cellsize = cubesize / N  # discretisation in all three nvdimensions.
        lex = cubesize / L  # exchange length.

        Km = 1e6  # magnetostatic energy density (J/m**3)
        Ms = np.sqrt(2 * Km / mm.consts.mu0)  # magnetisation saturation (A/m)
        A = 0.5 * mm.consts.mu0 * Ms**2 * lex**2  # exchange energy constant
        K = 0.1 * Km  # Uniaxial anisotropy constant
        u = (0, 0, 1)  # Uniaxial anisotropy easy-axis

        p1 = (0, 0, 0)  # Minimum sample coordinate.
        p2 = (cubesize, cubesize, cubesize)  # Maximum sample coordinate.
        cell = (cellsize, cellsize, cellsize)  # Discretisation.
        region = df.Region(p1=p1, p2=p2)
        mesh = df.Mesh(region=region, cell=cell)

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A) + mm.UniaxialAnisotropy(K=K, u=u) + mm.Demag()
        system.m = df.Field(mesh, nvdim=3, value=m_init, norm=Ms)

        if hasattr(calculator, "RelaxDriver"):
            system.dynamics = mm.Damping(alpha=0.5)
            md = calculator.RelaxDriver()
        else:
            md = calculator.MinDriver()
        md.drive(system)

        calculator.delete(system)

        return system

    def energy_difference(L):
        vortex = minimise_system_energy(L, m_init_vortex)
        flower = minimise_system_energy(L, m_init_flower)

        Evortex = vortex.table.data.tail(1)["E"][0]
        Eflower = flower.table.data.tail(1)["E"][0]

        return Evortex - Eflower

    cross_section = bisect(energy_difference, 8, 9, xtol=0.1)
    assert 8.4 < cross_section < 8.5



import random

import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestDMI:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-100e-9, 0, 0)
        p2 = (100e-9, 1e-9, 1e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.subregions = {
            "r1": df.Region(p1=(-100e-9, 0, 0), p2=(0, 1e-9, 1e-9)),
            "r2": df.Region(p1=(0, 0, 0), p2=(100e-9, 1e-9, 1e-9)),
        }

    def random_m(self, pos):
        return [2 * random.random() - 1 for i in range(3)]

    def test_scalar(self):
        name = "dmi_scalar"

        D = 1e-3
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.DMI(D=D, crystalclass="Cnv_z")

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=self.random_m, norm=Ms)

        if hasattr(self.calculator, "RelaxDriver"):
            system.dynamics = mm.Damping(alpha=0.5)
            md = self.calculator.RelaxDriver()
            with pytest.raises(RuntimeError):
                md.drive(system)
            system.energy += mm.Exchange(A=1e-21)
        else:
            md = self.calculator.MinDriver()

        md.drive(system)

        # There are 4N cells in the mesh. Because of that the average should be
        # 0.
        assert np.linalg.norm(system.m.mean()) < 1

        self.calculator.delete(system)

    def test_dict(self):
        name = "dmi_dict"

        D = {"r1": 0, "r2": 1e-3, "default": 2e-3}
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.DMI(D=D, crystalclass="Cnv_z")

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=self.random_m, norm=Ms)

        if hasattr(self.calculator, "RelaxDriver"):
            system.dynamics = mm.Damping(alpha=0.5)
            md = self.calculator.RelaxDriver()
            with pytest.raises(RuntimeError):
                md.drive(system)
            system.energy += mm.Exchange(A=1e-21)
        else:
            md = self.calculator.MinDriver()
        md.drive(system)

        assert np.linalg.norm(system.m["r1"].mean()) > 1
        # There are 4N cells in the region with D!=0. Because of that
        # the average should be 0.
        assert np.linalg.norm(system.m["r2"].mean()) < 1

        self.calculator.delete(system)

    def test_crystalclass(self):
        name = "dmi_crystalclass"

        D = 1e-3
        Ms = 1e6

        for crystalclass in [
            "Cnv_x",
            "Cnv_y",
            "Cnv_z",
            "T",
            "O",
            "D2d_x",
            "D2d_y",
            "D2d_z",
            "Cnv",
            "D2d",  # legacy crystalclass names
        ]:
            system = mm.System(name=name)
            system.energy = mm.DMI(D=D, crystalclass=crystalclass)

            if crystalclass.endswith(("x", "y")):
                mesh = df.Mesh(
                    p1=(0, 0, -100e-9), p2=(1e-9, 1e-9, 100e-9), cell=self.cell
                )
            else:
                mesh = df.Mesh(region=self.region, cell=self.cell)

            system.m = df.Field(mesh, nvdim=3, value=self.random_m, norm=Ms)

            md = self.calculator.MinDriver()
            md.drive(system)

            # There are 4N cells in the mesh. Because of that the
            # average should be 0.
            assert np.linalg.norm(system.m.mean()) < 1

        self.calculator.delete(system)

    def test_crystalclass_init(self):
        name = "dmi_crystalclass"

        D = 1e-3
        Ms = 1e6

        mumax3_cc = ["Cnv_z", "T", "O", "Cnv"]

        for crystalclass in [
            "Cnv_x",
            "Cnv_y",
            "Cnv_z",
            "T",
            "O",
            "D2d_x",
            "D2d_y",
            "D2d_z",
            "Cnv",
            "D2d",  # legacy crystalclass names
        ]:
            system = mm.System(name=name)
            system.energy = mm.DMI(D=D, crystalclass=crystalclass)

            if crystalclass.endswith(("x", "y")):
                mesh = df.Mesh(
                    p1=(0, 0, -100e-9), p2=(1e-9, 1e-9, 100e-9), cell=self.cell
                )
            else:
                mesh = df.Mesh(region=self.region, cell=self.cell)

            system.m = df.Field(mesh, nvdim=3, value=self.random_m, norm=Ms)

            md = self.calculator.MinDriver()
            if hasattr(self.calculator, "RelaxDriver"):
                system.energy += mm.Exchange(A=1e-21)
                if crystalclass not in mumax3_cc:
                    with pytest.raises(ValueError):
                        md.drive(system)
                else:
                    md.drive(system)
            else:
                md.drive(system)

        self.calculator.delete(system)



import inspect

import micromagnetictests as mt


def get_tests():
    """Generator yielding all available test names.

    Returns
    -------
    Generator

        Tests that can be imported from ``micromagnetictests``. The list
        consists of tuples, where the first element is the name of the tests,
        whereas the second element is the test object (function or class).

    Examples
    --------
    1. Getting the list of available tests.

    >>> import micromagnetictests as mt
    ...
    >>> list(mt.get_tests())
    [...]

    """
    for name, object in inspect.getmembers(mt.calculatortests):
        if inspect.isclass(object) or inspect.isfunction(object):
            starting_strings = ["__", "test_", "Test"]
            if any([name.startswith(s) for s in starting_strings]):
                yield (name, object)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestExchange:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-5e-9, -5e-9, -3e-9)
        p2 = (5e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.n = (10, 10, 6)
        self.subregions = {
            "r1": df.Region(p1=(-5e-9, -5e-9, -3e-9), p2=(5e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-5e-9, 0, -3e-9), p2=(5e-9, 5e-9, 3e-9)),
        }

    def m_init(self, pos):
        x, y, z = pos
        if y <= 0:
            return (0, 0.2, 1)
        else:
            return (0, -0.7, -0.4)

    def test_scalar(self):
        name = "exchange_scalar"

        A = 1e-12
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A)

        mesh = df.Mesh(region=self.region, n=self.n)
        system.m = df.Field(mesh, nvdim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        assert abs(np.linalg.norm(system.m.mean()) - Ms) < 1

        self.calculator.delete(system)

    def test_dict(self):
        name = "exchange_dict"

        A = {"r1": 3e-12, "r2": 1e-12, "r1:r2": -1e-12}
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A)

        mesh = df.Mesh(region=self.region, n=self.n, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        # r1
        assert abs(np.linalg.norm(system.m["r1"].mean()) - Ms) < 1
        # r2
        assert abs(np.linalg.norm(system.m["r2"].mean()) - Ms) < 1

        assert (
            abs(
                np.dot(
                    system.m["r1"].orientation.mean(),
                    system.m["r2"].orientation.mean(),
                )
                - (-1)
            )
            < 1e-3
        )

        self.calculator.delete(system)

    def test_field(self):
        name = "exchange_field"

        def A_fun(pos):
            x, y, z = pos
            if x <= 0:
                return 1e-10  # for 0, OOMMF gives nan
            else:
                return 1e-12

        mesh = df.Mesh(region=self.region, n=self.n)
        A = df.Field(mesh, nvdim=1, value=A_fun)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Exchange(A=A)

        system.m = df.Field(mesh, nvdim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        assert abs(np.linalg.norm(system.m.mean()) - Ms) < 1

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestHysteresisDriver:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (0, 0, 0)
        p2 = (5e-9, 5e-9, 5e-9)
        n = (5, 5, 5)
        self.Ms = 1e6
        A = 1e-12
        H = (0, 0, 1e6)
        region = df.Region(p1=p1, p2=p2)
        self.mesh = df.Mesh(region=region, n=n)
        self.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
        self.m = df.Field(self.mesh, nvdim=3, value=(0, 1, 0), norm=self.Ms)

    def test_noevolver_nodriver(self):
        name = "hysteresisdriver_noevolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.m = self.m

        hd = self.calculator.HysteresisDriver()
        hd.drive(system, Hmin=(0, 0, -1e6), Hmax=(0, 0, 1e6), n=3)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1e-3

        assert len(system.table.data.index) == 5

        assert system.table.x == "B_hysteresis"

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestDynamics:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-5e-9, -5e-9, -3e-9)
        p2 = (5e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.n = (10, 10, 10)
        self.subregions = {
            "r1": df.Region(p1=(-5e-9, -5e-9, -3e-9), p2=(5e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-5e-9, 0, -3e-9), p2=(5e-9, 5e-9, 3e-9)),
        }

    def test_scalar_scalar(self):
        name = "dynamics_scalar_scalar"

        H = (0, 0, 1e6)
        alpha = 1
        gamma0 = 2.211e5
        Ms = 1e6

        mesh = df.Mesh(region=self.region, n=self.n)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Precession(gamma0=gamma0) + mm.Damping(alpha=alpha)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # Alpha is zero, nothing should change.
        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(np.divide(value, Ms), (0, 0, 1))) < 1e-5

        self.calculator.delete(system)

    def test_scalar_dict(self):
        name = "dynamics_scalar_dict"

        H = (0, 0, 1e6)
        gamma0 = 2.211e5
        alpha = {"r1": 0, "r2": 1}
        Ms = 1e6

        mesh = df.Mesh(region=self.region, n=self.n, subregions=self.subregions)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Precession(gamma0=gamma0) + mm.Damping(alpha=alpha)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # alpha=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0, Ms))) > 1

        # alpha!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(np.divide(value, Ms), (0, 0, 1))) < 1e-5

        self.calculator.delete(system)

    def test_field_field(self):
        name = "dynamics_field_field"

        mesh = df.Mesh(region=self.region, n=self.n)

        def alpha_fun(pos):
            x, y, z = pos
            if y <= 0:
                return 0
            else:
                return 1

        def gamma0_fun(pos):
            x, y, z = pos
            if y <= 0:
                return 0
            else:
                return 2.211e5

        H = (0, 0, 1e6)
        alpha = df.Field(mesh, nvdim=1, value=alpha_fun)
        gamma0 = df.Field(mesh, nvdim=1, value=gamma0_fun)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.Precession(gamma0=gamma0) + mm.Damping(alpha=alpha)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # alpha=0 and gamma=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # alpha!=0 and gamma!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(np.divide(value, Ms), (0, 0, 1))) < 1e-5

        self.calculator.delete(system)



import sys

import discretisedfield as df
import micromagneticmodel as mm
import pytest


class TestCompute:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        name = "compute_tests"
        p1 = (0, 0, 0)
        p2 = (10e-9, 2e-9, 2e-9)
        cell = (2e-9, 2e-9, 2e-9)
        region = df.Region(p1=p1, p2=p2)
        subregions = {
            "a": df.Region(p1=(0, 0, 0), p2=(6e-9, 2e-9, 2e-9)),
            "b": df.Region(p1=(6e-9, 0, 0), p2=(10e-9, 2e-9, 2e-9)),
        }
        mesh = df.Mesh(region=region, cell=cell, subregions=subregions)
        self.subregions = subregions
        self.system = mm.System(name=name)
        self.system.energy = (
            mm.Exchange(A=1e-12)
            + mm.Demag()
            + mm.Zeeman(H=(8e6, 0, 0))
            + mm.UniaxialAnisotropy(K=1e4, u=(0, 0, 1))
            + mm.CubicAnisotropy(K=1e3, u1=(1, 0, 0), u2=(0, 1, 0))
        )

        self.system.m = df.Field(mesh, nvdim=3, value=(0, 0, 1), norm=8e6)

    def test_energy(self):
        for term in self.system.energy:
            assert isinstance(self.calculator.compute(term.energy, self.system), float)
        assert isinstance(
            self.calculator.compute(self.system.energy.energy, self.system), float
        )
        self.calculator.delete(self.system)

    def test_energy_density(self):
        for term in self.system.energy:
            e_density = self.calculator.compute(term.density, self.system)
            assert isinstance(e_density, df.Field)
            assert e_density.mesh.subregions == self.subregions
        e_density = self.calculator.compute(self.system.energy.density, self.system)
        assert isinstance(e_density, df.Field)
        assert e_density.mesh.subregions == self.subregions
        self.calculator.delete(self.system)

    def test_effective_field(self):
        for term in self.system.energy:
            effective_field = self.calculator.compute(term.effective_field, self.system)
            assert isinstance(effective_field, df.Field)
            assert effective_field.mesh.subregions == self.subregions
        effective_field = self.calculator.compute(
            self.system.energy.effective_field, self.system
        )
        assert isinstance(effective_field, df.Field)
        assert effective_field.mesh.subregions == self.subregions
        self.calculator.delete(self.system)

    def test_invalid_func(self):
        with pytest.raises(ValueError):
            self.calculator.compute(self.system.energy.__len__, self.system)

    def test_dmi(self):
        if sys.platform != "win32":
            self.system.energy += mm.DMI(D=5e-3, crystalclass="T")
            term = self.system.energy.dmi
            for crystalclass in [
                "T",
                "Cnv_x",
                "Cnv_y",
                "Cnv_z",
                "D2d_x",
                "D2d_y",
                "D2d_z",
            ]:
                term.crystalclass = crystalclass
                assert isinstance(
                    self.calculator.compute(term.energy, self.system), float
                )
                e_density = self.calculator.compute(term.density, self.system)
                assert isinstance(e_density, df.Field)
                assert e_density.mesh.subregions == self.subregions
                effective_field = self.calculator.compute(
                    term.effective_field, self.system
                )
                assert isinstance(effective_field, df.Field)
                assert effective_field.mesh.subregions == self.subregions
            assert isinstance(
                self.calculator.compute(self.system.energy.energy, self.system), float
            )
            self.calculator.delete(self.system)

    def test_slonczewski(self):
        self.system.dynamics = mm.Slonczewski(J=7.5e12, mp=(1, 0, 0), P=0.4, Lambda=2)
        assert isinstance(
            self.calculator.compute(self.system.energy.energy, self.system), float
        )
        self.calculator.delete(self.system)

    def test_zhang_li(self):
        self.system.dynamics = mm.ZhangLi(beta=0.01, u=5e6)
        assert isinstance(
            self.calculator.compute(self.system.energy.energy, self.system), float
        )
        self.calculator.delete(self.system)



import sys

import micromagneticmodel as mm
import pytest


@pytest.mark.skipif(
    sys.platform == "win32",
    reason="Calling the oommf executable without a full path does not properly work.",
)
def test_schedule(calculator, tmp_path):
    system = mm.examples.macrospin()

    td = calculator.TimeDriver()
    with pytest.raises(RuntimeError):
        # We have no test system with a job scheduling system such as slurm.
        # Instead, we use oommf to test that the mif file creation and the
        # subprocess call to schedule the run work as expected.
        # OOMMF will raise an error because we call it with a "command" job.sh (the
        # submission system script)
        td.schedule(
            system,
            "oommf",
            "scheduling resources",
            dirname=str(tmp_path),
            t=0.2e-9,
            n=50,
        )
        assert len(list(tmp_path.glob("**/job.sh"))) == 1



"""Calculator tests"""
from .compute import TestCompute
from .cubicanisotropy import TestCubicAnisotropy
from .damping import TestDamping
from .demag import TestDemag
from .dirname import test_dirname
from .dmi import TestDMI
from .dynamics import TestDynamics
from .energy import TestEnergy
from .exchange import TestExchange
from .fixedsubregions import TestFixedSubregions
from .hysteresisdriver import TestHysteresisDriver
from .info_file import test_info_file
from .mesh import TestMesh
from .mindriver import TestMinDriver
from .multiple_drives import test_multiple_drives
from .outputformat import test_format
from .outputstep import test_outputstep
from .precession import TestPrecession
from .rkky import TestRKKY
from .schedule import test_schedule
from .skyrmion import test_skyrmion
from .slonczewski import TestSlonczewski
from .stdprob3 import test_stdprob3
from .stdprob4 import test_stdprob4
from .stdprob5 import test_stdprob5
from .threads import TestThreads
from .timedriver import TestTimeDriver
from .uniaxialanisotropy import TestUniaxialAnisotropy
from .zeeman import TestZeeman
from .zhangli import TestZhangLi



import os
import shutil

import discretisedfield as df
import micromagneticmodel as mm


def test_dirname(calculator):
    name = "specifying_dirname"
    mydirname = "./my_dirname"

    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    n = (2, 2, 2)
    Ms = 1e6
    A = 1e-12
    H = (0, 0, 1e6)
    region = df.Region(p1=p1, p2=p2)
    mesh = df.Mesh(region=region, n=n)

    system = mm.System(name=name)
    system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
    system.dynamics = mm.Precession(gamma0=mm.consts.gamma0) + mm.Damping(alpha=1)
    system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

    md = calculator.MinDriver()
    md.drive(system, dirname=mydirname)

    dirname = os.path.join(mydirname, name, "drive-0")
    assert os.path.exists(dirname)

    system.energy.zeeman.H = (1e6, 0, 0)

    td = calculator.TimeDriver()
    td.drive(system, t=100e-12, n=10, dirname=mydirname)

    dirname = os.path.join(mydirname, name, "drive-1")
    assert os.path.exists(dirname)

    shutil.rmtree(mydirname)



import glob
import os

import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestTimeDriver:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (0, 0, 0)
        p2 = (5e-9, 5e-9, 5e-9)
        n = (2, 2, 2)
        self.Ms = 1e6
        A = 1e-12
        H = (0, 0, 1e6)
        region = df.Region(p1=p1, p2=p2)
        self.mesh = df.Mesh(region=region, n=n)
        self.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
        self.precession = mm.Precession(gamma0=mm.consts.gamma0)
        self.damping = mm.Damping(alpha=1)
        self.m = df.Field(self.mesh, nvdim=3, value=(0, 0.1, 1), norm=self.Ms)

    def test_noevolver_nodriver(self):
        name = "timedriver_noevolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 10

        assert system.table.x == "t"

        self.calculator.delete(system)

    def test_rungekutta_evolver_nodriver(self):
        name = "timedriver_rungekutta_evolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m

        evolver = self.calculator.RungeKuttaEvolver(method="rkf54s")
        td = self.calculator.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        self.calculator.delete(system)

    def test_euler_evolver_nodriver(self):
        name = "timedriver_euler_evolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m

        evolver = self.calculator.EulerEvolver(start_dm=0.02)
        td = self.calculator.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        self.calculator.delete(system)

    def test_theta_evolver_nodriver(self):
        name = "timedriver_theta_evolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m
        system.T = 10

        evolver = self.calculator.UHH_ThetaEvolver(fixed_timestep=2e-13)
        td = self.calculator.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        self.calculator.delete(system)

    def test_therm_heun_evolver_nodriver(self):
        name = "timedriver_therm_heun_evolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m
        system.T = 10

        evolver = self.calculator.Xf_ThermHeunEvolver()
        td = self.calculator.TimeDriver(evolver=evolver)
        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        self.calculator.delete(system)

    def test_noevolver_nodriver_finite_temperature(self):
        name = "timedriver_therm_heun_evolver_nodriver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m
        system.T = 10

        td = self.calculator.TimeDriver()
        with pytest.raises(RuntimeError):
            td.drive(system, t=0.2e-9, n=50)

    def test_noevolver_driver(self):
        name = "timedriver_noevolver_driver"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = self.calculator.TimeDriver(stopping_dm_dt=0.01)
        td.drive(system, t=0.3e-9, n=50)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 1

        self.calculator.delete(system)

    def test_noprecession(self):
        name = "timedriver_noprecession"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.damping
        system.m = self.m

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) < 10

        self.calculator.delete(system)

    def test_nodamping(self):
        name = "timedriver_nodamping"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession
        system.m = self.m

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        value = system.m(self.mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, self.Ms))) > 1e3

        self.calculator.delete(system)

    def test_output_files(self):
        name = "timedriver_output_files"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50, save=True, overwrite=True)

        dirname = os.path.join(f"{name}", f"drive-{system.drive_number-1}")
        assert os.path.exists(dirname)
        if os.path.exists(os.path.join(dirname, f"{name}.out")):
            mumax3_path = os.path.join(dirname, f"{name}.out")
            mx3filename = os.path.join(dirname, f"{name}.mx3")
            assert os.path.isfile(mx3filename)
            omffilename = os.path.join(dirname, "m0.omf")
            assert os.path.isfile(omffilename)
            omf_files = list(glob.iglob(os.path.join(mumax3_path, "*.ovf")))
            assert len(omf_files) == 50
        else:
            miffilename = os.path.join(dirname, f"{name}.mif")
            assert os.path.isfile(miffilename)
            omf_files = list(glob.iglob(os.path.join(dirname, "*.omf")))
            assert len(omf_files) == 51
            odt_files = list(glob.iglob(os.path.join(dirname, "*.odt")))
            assert len(odt_files) == 1
            omffilename = os.path.join(dirname, "m0.omf")
            assert omffilename in omf_files

        self.calculator.delete(system)

    def test_drive_exception(self):
        name = "timedriver_exception"

        system = mm.System(name=name)
        system.energy = self.energy
        system.dynamics = self.precession + self.damping
        system.m = self.m

        td = self.calculator.TimeDriver()
        with pytest.raises(ValueError):
            td.drive(system, t=-0.1e-9, n=10)
        with pytest.raises(ValueError):
            td.drive(system, t=0.1e-9, n=-10)

    def test_wrong_evolver(self):
        system = mm.examples.macrospin()
        evolver = self.calculator.CGEvolver()
        td = self.calculator.TimeDriver(evolver=evolver)

        with pytest.raises(TypeError):
            td.drive(system, t=1e-12, n=1)

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestZeeman:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-10e-9, -5e-9, -3e-9)
        p2 = (10e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.subregions = {
            "r1": df.Region(p1=(-10e-9, -5e-9, -3e-9), p2=(10e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-10e-9, 0, -3e-9), p2=(10e-9, 5e-9, 3e-9)),
        }

    def test_vector(self):
        name = "zeeman_vector"

        H = (0, 0, 1e6)
        Ms = 1e6

        system = mm.System(name=name)

        # time-independent
        system.energy = mm.Zeeman(H=H)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

    def test_time_vector(self):
        name = "zeeman_vector"

        H = (0, 0, 1e6)
        Ms = 1e6

        system = mm.System(name=name)

        # time-independent
        system.energy = mm.Zeeman(H=H)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        # time-dependent - sin
        system.energy = mm.Zeeman(H=H, func="sin", f=1e9, t0=1e-12)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - sinc
        system.energy = mm.Zeeman(H=H, func="sinc", f=1e9, t0=0)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        self.calculator.delete(system)

        # time-dependent - function
        def t_func(t):
            if t < 1e-10:
                return 1
            elif t < 5e-10:
                return (5e-10 - t) / 4e-10
            else:
                return 0

        system.energy = mm.Zeeman(H=H, func=t_func, dt=1e-13)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - two terms
        f = 10e9

        def cos_wave(t):
            return np.cos(2 * np.pi * f * t)

        def sin_wave(t):
            return np.sin(2 * np.pi * f * t)

        H_x = (1e6, 0, 0)
        H_y = (0, 1e6, 0)
        system.energy = mm.Zeeman(
            H=H_x, func=cos_wave, dt=5e-12, name="xdir"
        ) + mm.Zeeman(H=H_y, func="sin", t0=0, f=f, name="ydir")
        td.drive(system, t=0.1e-9, n=100)

        assert not np.allclose(system.table.data["Bx_xdir"], 0)
        assert np.allclose(system.table.data["By_xdir"], 0)

        assert np.allclose(system.table.data["Bx_ydir"], 0)
        assert not np.allclose(system.table.data["By_ydir"], 0)

        assert np.isclose(
            np.max(system.table.data["Bx_xdir"]), np.max(system.table.data["By_ydir"])
        )

        H_x = (1e6, 0, 0)
        H_y = (0, 1e6, 0)
        system.energy = mm.Zeeman(
            H=H_x, func=cos_wave, dt=5e-12, name="xdir"
        ) + mm.Zeeman(H=H_y, func=sin_wave, dt=5e-12, name="ydir")
        td.drive(system, t=0.1e-9, n=100)

        assert not np.allclose(system.table.data["Bx_xdir"], 0)
        assert np.allclose(system.table.data["By_xdir"], 0)

        assert np.allclose(system.table.data["Bx_ydir"], 0)
        assert not np.allclose(system.table.data["By_ydir"], 0)

        assert np.isclose(
            np.max(system.table.data["Bx_xdir"]), np.max(system.table.data["By_ydir"])
        )

        # time-dependent - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            set Hx [expr {sin($total_time * 1e10)}]
            set dHx [expr {1e10 * cos($total_time * 1e10)}]
            return [list $Hx 0 0 $dHx 0 0]
        }
        """
        tcl_strings["energy"] = "Oxs_ScriptUZeeman"
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.energy = mm.Zeeman(H=H, tcl_strings=tcl_strings)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        self.calculator.delete(system)

    def test_dict(self):
        name = "zeeman_dict"

        H = {"r1": (1e5, 0, 0), "r2": (0, 0, 1e5)}
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        assert np.linalg.norm(np.subtract(system.m["r1"].mean(), (Ms, 0, 0))) < 1

        assert np.linalg.norm(np.subtract(system.m["r2"].mean(), (0, 0, Ms))) < 1

    def test_time_dict(self):
        name = "zeeman_dict"

        H = {"r1": (1e5, 0, 0), "r2": (0, 0, 1e5)}
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        assert np.linalg.norm(np.subtract(system.m["r1"].mean(), (Ms, 0, 0))) < 1

        assert np.linalg.norm(np.subtract(system.m["r2"].mean(), (0, 0, Ms))) < 1

        # time-dependent - sin
        system.energy = mm.Zeeman(H=H, func="sin", f=1e9, t0=1e-12)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - sinc
        system.energy = mm.Zeeman(H=H, func="sinc", f=1e9, t0=0)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - function
        def t_func(t):
            if t < 1e-10:
                return 1
            elif t < 5e-10:
                return (5e-10 - t) / 4e-10
            else:
                return 0

        system.energy = mm.Zeeman(H=H, func=t_func, dt=1e-13)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        self.calculator.delete(system)

    def test_field(self):
        name = "zeeman_field"

        def value_fun(pos):
            x, y, z = pos
            if x <= 0:
                return (1e6, 0, 0)
            else:
                return (0, 0, 1e6)

        mesh = df.Mesh(region=self.region, cell=self.cell)

        H = df.Field(mesh, nvdim=3, value=value_fun)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(0, 1, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m((-2e-9, -2e-9, -2e-9))
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        value = system.m((2e-9, 2e-9, 2e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

    def test_time_field(self):
        name = "zeeman_field"

        def value_fun(pos):
            x, y, z = pos
            if x <= 0:
                return (1e6, 0, 0)
            else:
                return (0, 0, 1e6)

        mesh = df.Mesh(region=self.region, cell=self.cell)

        H = df.Field(mesh, nvdim=3, value=value_fun)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.m = df.Field(mesh, nvdim=3, value=(0, 1, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        value = system.m((-2e-9, -2e-9, -2e-9))
        assert np.linalg.norm(np.subtract(value, (Ms, 0, 0))) < 1e-3

        value = system.m((2e-9, 2e-9, 2e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) < 1e-3

        # time-dependent - sin
        system.energy = mm.Zeeman(H=H, func="sin", f=1e9, t0=1e-12)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - sinc
        system.energy = mm.Zeeman(H=H, func="sinc", f=1e9, t0=0)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - function
        def t_func(t):
            omega = 2 * np.pi * 1e9
            return [
                np.cos(omega * t),
                -np.sin(omega * t),
                0,
                np.sin(omega * t),
                np.cos(omega * t),
                0,
                0,
                0,
                1,
            ]

        system.energy = mm.Zeeman(H=H, func=t_func, dt=1e-13)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        # time-dependent - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            set PI [expr {4*atan(1.)}]
            set w [expr {1e9*2*$PI}]
            set ct [expr {cos($w*$total_time)}]
            set mct [expr {-1*$ct}]      ;# "mct" is "minus cosine (w)t"
            set st [expr {sin($w*$total_time)}]
            set mst [expr {-1*$st}]      ;# "mst" is "minus sine (w)t"
            return [list  $ct $mst  0 \
                          $st $ct   0 \
                          0   0   1 \
                          [expr {$w*$mst}] [expr {$w*$mct}] 0 \
                          [expr {$w*$ct}]  [expr {$w*$mst}] 0 \
                              0                0         0]
        }"""
        tcl_strings["energy"] = "Oxs_TransformZeeman"
        tcl_strings["type"] = "general"
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.energy = mm.Zeeman(H=H, tcl_strings=tcl_strings)

        mesh = df.Mesh(region=self.region, cell=self.cell)
        system.m = df.Field(mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.1e-9, n=20)

        self.calculator.delete(system)



import os

import discretisedfield as df
import micromagneticmodel as mm


def test_multiple_drives(calculator):
    name = "multiple_drives"

    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    n = (2, 2, 2)
    Ms = 1e6
    A = 1e-12
    H = (0, 0, 1e6)
    region = df.Region(p1=p1, p2=p2)
    mesh = df.Mesh(region=region, n=n)

    system = mm.System(name=name)
    system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
    system.dynamics = mm.Precession(gamma0=mm.consts.gamma0) + mm.Damping(alpha=1)
    system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

    md = calculator.MinDriver()
    md.drive(system)

    dirname = os.path.join(name, "drive-0")
    assert os.path.exists(dirname)

    system.energy.zeeman.H = (1e6, 0, 0)

    td = calculator.TimeDriver()
    td.drive(system, t=100e-12, n=10)

    dirname = os.path.join(name, "drive-1")
    assert os.path.exists(dirname)

    # assert len(os.listdir(name)) == 2

    calculator.delete(system)


def test_multiple_drives_compute(calculator):
    name = "multiple_drives"

    p1 = (0, 0, 0)
    p2 = (5e-9, 5e-9, 5e-9)
    n = (2, 2, 2)
    Ms = 1e6
    A = 1e-12
    H = (0, 0, 1e6)
    region = df.Region(p1=p1, p2=p2)
    mesh = df.Mesh(region=region, n=n)

    system = mm.System(name=name)
    system.energy = mm.Exchange(A=A) + mm.Zeeman(H=H)
    system.dynamics = mm.Precession(gamma0=mm.consts.gamma0) + mm.Damping(alpha=1)
    system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

    calculator.compute(system.energy.zeeman.energy, system)

    dirname = os.path.join(name, "compute-0")
    assert os.path.exists(dirname)

    calculator.compute(system.energy.zeeman.effective_field, system)

    dirname = os.path.join(name, "compute-1")
    assert os.path.exists(dirname)

    assert len(os.listdir(name)) == 2

    calculator.delete(system)



"""Test computational magnetism tools."""
import importlib.metadata

import pytest

import micromagnetictests.calculatortests

from .get_tests import get_tests

__version__ = importlib.metadata.version(__package__)


def test():
    """Run all package tests.

    Examples
    --------
    1. Run all tests.

    >>> import micromagnetictests as mt
    ...
    >>> # mt.test()

    """
    return pytest.main(
        ["-v", "--pyargs", "micromagnetictests", "-l"]
    )  # pragma: no cover



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestFixedSubregions:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-10e-9, -5e-9, -3e-9)
        p2 = (10e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.subregions = {
            "r1": df.Region(p1=(-10e-9, -5e-9, -3e-9), p2=(10e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-10e-9, 0, -3e-9), p2=(10e-9, 5e-9, 3e-9)),
        }

    def test_fixed_subregions(self):
        name = "fixed_subregions"

        H = (0, 0, 1e5)
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=(1, 0, 0), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system, fixed_subregions=["r1"])

        assert np.linalg.norm(np.subtract(system.m["r1"].mean(), (Ms, 0, 0))) < 1

        assert np.linalg.norm(np.subtract(system.m["r2"].mean(), (0, 0, Ms))) < 1

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestRKKY:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-5e-9, -5e-9, -3e-9)
        p2 = (5e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.n = (10, 10, 6)
        self.subregions = {
            "r1": df.Region(p1=(-5e-9, -5e-9, -3e-9), p2=(5e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-5e-9, 0, -3e-9), p2=(5e-9, 1e-9, 3e-9)),
            "r3": df.Region(p1=(-5e-9, 1e-9, -3e-9), p2=(5e-9, 5e-9, 3e-9)),
        }

    def m_init(self, pos):
        x, y, z = pos
        if y <= 0:
            return (0, 0.2, 1)
        else:
            return (0, -0.5, -1)

    def test_scalar(self):
        name = "rkky_scalar"

        sigma = -1e4
        sigma2 = 0
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.RKKY(sigma=sigma, sigma2=sigma2, subregions=["r1", "r3"])

        mesh = df.Mesh(region=self.region, n=self.n, subregions=self.subregions)
        system.m = df.Field(mesh, nvdim=3, value=self.m_init, norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        # AFM
        m1 = system.m.orientation((0, -0.5e-9, 0))
        m2 = system.m.orientation((0, 1.5e-9, 0))
        assert abs(np.dot(m1, m2) - (-1)) < 1e-3

        system.energy.rkky.sigma = 1e4
        system.energy.rkky.sigma2 = 0

        system.m = df.Field(mesh, nvdim=3, value=self.m_init, norm=Ms)

        md.drive(system)

        # FM
        m1 = system.m.orientation((0, -0.5e-9, 0))
        m2 = system.m.orientation((0, 1.5e-9, 0))
        assert abs(np.dot(m1, m2) - 1) < 1e-3

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import numpy as np
import pytest


class TestZhangLi:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-5e-9, -5e-9, -3e-9)
        p2 = (5e-9, 5e-9, 3e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.cell = (1e-9, 1e-9, 3e-9)
        self.subregions = {
            "r1": df.Region(p1=(-5e-9, -5e-9, -3e-9), p2=(5e-9, 0, 3e-9)),
            "r2": df.Region(p1=(-5e-9, 0, -3e-9), p2=(5e-9, 5e-9, 3e-9)),
        }

    def test_scalar_scalar(self):
        name = "zhangli_scalar_scalar"

        u = 0
        beta = 0.5
        H = (0, 0, 1e5)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, cell=self.cell)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.ZhangLi(u=u, beta=beta)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # u is zero, nothing should change.
        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        system.dynamics -= mm.ZhangLi(u=u, beta=beta)
        td.drive(system, t=0.2e-9, n=50)

    def test_time_scalar_scalar(self):
        name = "zhangli_scalar_scalar"

        u = 0
        beta = 0.5
        H = (0, 0, 1e5)
        Ms = 1e6

        mesh = df.Mesh(region=self.region, cell=self.cell)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.ZhangLi(u=u, beta=beta)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # u is zero, nothing should change.
        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        system.dynamics -= mm.ZhangLi(u=u, beta=beta)
        td.drive(system, t=0.2e-9, n=50)

        # Check if it runs.

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.ZhangLi(u=u, beta=beta, func=time_dep, dt=1e-13)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # u is zero, nothing should change.
        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.ZhangLi(u=u, beta=beta, tcl_strings=tcl_strings)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # u is zero, nothing should change.
        value = system.m(mesh.region.random_point())
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        self.calculator.delete(system)

    def test_dict_scalar(self):
        name = "zhangli_dict_scalar"

        H = (0, 0, 1e6)
        u = {"r1": 0, "r2": 1}
        beta = 0.5
        Ms = 1e6

        mesh = df.Mesh(region=self.region, cell=self.cell, subregions=self.subregions)

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.ZhangLi(u=u, beta=beta)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # u=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # u!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) > 1

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.ZhangLi(u=u, beta=beta, func=time_dep, dt=1e-13)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # u=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # u!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) > 1

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.ZhangLi(u=u, beta=beta, tcl_strings=tcl_strings)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # u=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # u!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) > 1

        self.calculator.delete(system)

    def test_field_scalar(self):
        name = "zhangli_field_scalar"

        mesh = df.Mesh(region=self.region, cell=self.cell)

        def u_fun(pos):
            x, y, z = pos
            if y <= 0:
                return 0
            else:
                return 1

        H = (0, 0, 1e6)
        u = df.Field(mesh, nvdim=1, value=u_fun)
        beta = 0.5
        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Zeeman(H=H)
        system.dynamics = mm.ZhangLi(u=u, beta=beta)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td = self.calculator.TimeDriver()
        td.drive(system, t=0.2e-9, n=50)

        # u=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # u!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) > 1

        # time-dependence - function
        def time_dep(t):
            return np.sin(t * 1e10)

        system.dynamics = mm.ZhangLi(u=u, beta=beta, func=time_dep, dt=1e-13)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # u=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # u!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) > 1

        # time-dependence - tcl strings
        tcl_strings = {}
        tcl_strings[
            "script"
        ] = """proc TimeFunction { total_time } {
            return $total_time
        }
        """
        tcl_strings["script_args"] = "total_time"
        tcl_strings["script_name"] = "TimeFunction"

        system.dynamics = mm.ZhangLi(u=u, beta=beta, tcl_strings=tcl_strings)
        system.m = df.Field(mesh, nvdim=3, value=(0, 0.1, 1), norm=Ms)

        td.drive(system, t=0.2e-9, n=50)

        # u=0 region
        value = system.m((1e-9, -4e-9, 3e-9))
        assert np.linalg.norm(np.cross(value, (0, 0.1 * Ms, Ms))) < 1e-3

        # u!=0 region
        value = system.m((1e-9, 4e-9, 3e-9))
        assert np.linalg.norm(np.subtract(value, (0, 0, Ms))) > 1

        self.calculator.delete(system)



import discretisedfield as df
import micromagneticmodel as mm
import pytest


class TestDemag:
    @pytest.fixture(autouse=True)
    def _setup_calculator(self, calculator):
        self.calculator = calculator

    def setup_method(self):
        p1 = (-5e-9, 0, 0)
        p2 = (5e-9, 5e-9, 1e-9)
        self.cell = (1e-9, 1e-9, 1e-9)
        self.region = df.Region(p1=p1, p2=p2)
        self.mesh = df.Mesh(region=self.region, cell=self.cell)

    def test_demag(self):
        name = "demag"

        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Demag()

        system.m = df.Field(self.mesh, nvdim=3, value=(1, 1, 1), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        # Check if it runs. Tests to be added here.

        self.calculator.delete(system)

    def test_demag_asymptotic_radius(self):
        name = "demag_asymptotic_radius"

        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Demag(asymptotic_radius=6)

        system.m = df.Field(self.mesh, nvdim=3, value=(0, 0, 1), norm=Ms)

        md = self.calculator.MinDriver()
        md.drive(system)

        # Check if it runs. Tests to be added here.

        self.calculator.delete(system)

    def test_demag_1_pbc(self):
        name = "demag_pbc"

        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Demag()

        md = self.calculator.MinDriver()

        # 1D pbc
        mesh = df.Mesh(region=self.region, cell=self.cell, bc="x")
        system.m = df.Field(mesh, nvdim=3, value=(0, 0, 1), norm=Ms)

        md.drive(system)
        self.calculator.delete(system)

    def test_demag_2_pbc(self):
        name = "demag_pbc"

        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Demag()

        md = self.calculator.MinDriver()

        # 2D pbc
        mesh = df.Mesh(region=self.region, cell=self.cell, bc="xy")
        system.m = df.Field(mesh, nvdim=3, value=(0, 0, 1), norm=Ms)

        if not hasattr(self.calculator, "RelaxDriver"):
            with pytest.raises(ValueError):
                md.drive(system)
        else:
            md.drive(system)

        self.calculator.delete(system)

    def test_demag_3_pbc(self):
        name = "demag_pbc"

        Ms = 1e6

        system = mm.System(name=name)
        system.energy = mm.Demag()

        md = self.calculator.MinDriver()

        # 3D pbc
        mesh = df.Mesh(region=self.region, cell=self.cell, bc="xyz")
        system.m = df.Field(mesh, nvdim=3, value=(0, 0, 1), norm=Ms)

        if not hasattr(self.calculator, "RelaxDriver"):
            with pytest.raises(ValueError):
                md.drive(system)
        else:
            md.drive(system)

        self.calculator.delete(system)
